@@ -32,7 +32,9 @@ def handler(signal_received, frame):
 
 def convert_to_seconds(time_str):
     """Convert a time string to seconds.
-    time_str can be one of Nh, Nm or Ns, or of the form HH:MM:SS"""
+    time_str can be one of Nh, Nm or Ns, or of the form HH:MM:SS
+    """
+    logging.info("Converting '{}' to a number of seconds".format(time_str))
     time_str = time_str.lower()
     try:
         if ":" in time_str:  # Assume form is HH:MM:SS or HH:MM
@@ -66,10 +68,13 @@ def convert_to_seconds(time_str):
 def get_speaker(name, local=False):
     # Allow the use of an IP address even if 'local' is specified
     if speakers.Speakers.is_ipv4_address(name):
+        logging.info("Using IP address instead of speaker name")
         return soco.SoCo(name)
     if local:
+        logging.info("Using local speaker list")
         return speaker_list.find(name)
     else:
+        logging.info("Using SoCo speaker discovery")
         return soco.discovery.by_name(name)
 
 
@@ -143,7 +148,10 @@ def main():
 
     # Set up logging
     log_level = args.log.lower()
-    if log_level != "none":
+    if log_level == "none":
+        # Disables all logging (i.e., CRITICAL and below)
+        logging.disable(logging.CRITICAL)
+    else:
         log_format = (
             "%(asctime)s %(filename)s:%(lineno)s - %(funcName)s() - %(message)s"
         )
@@ -157,6 +165,10 @@ def main():
             logging.basicConfig(format=log_format, level=logging.ERROR)
         elif log_level == "critical":
             logging.basicConfig(format=log_format, level=logging.CRITICAL)
+        else:
+            error_and_exit(
+                "--log takes one of: NONE, DEBUG, INFO, WARN, ERROR, CRITICAL"
+            )
 
     use_local_speaker_list = args.use_local_speaker_list
     if use_local_speaker_list:
@@ -166,7 +178,7 @@ def main():
             network_timeout=args.network_discovery_timeout,
         )
         if args.refresh_local_speaker_list or not speaker_list.load():
-            logging.info("Discovering speakers")
+            logging.info("Start speaker discovery")
             speaker_list.discover()
             speaker_list.save()
 
@@ -196,7 +208,7 @@ def main():
         sequences.append(sequence)
 
     # Loop through processing command sequences
-    logging.info("Found {} action sequence(s)".format(len(sequences)))
+    logging.info("Found {} action sequence(s): {}".format(len(sequences), sequences))
     for sequence in sequences:
         try:
             if len(sequence) < 2:
@@ -215,7 +227,7 @@ def main():
                     time.sleep(duration)
                 else:
                     error_and_exit(
-                        "'wait' requires float number of hours, seconds or minutes + 'h/m/s'"
+                        "'wait' requires number hours, seconds or minutes + 'h/m/s', or HH:MM(:SS)"
                     )
                 continue
             elif speaker_name in ["wait_until"]:
@@ -227,7 +239,7 @@ def main():
                     time.sleep(duration)
                 except ValueError:
                     error_and_exit(
-                        "'wait_until' requires parameter: time in 24hr HH:MM format"
+                        "'wait_until' requires parameter: time in 24hr HH:MM(:SS) format"
                     )
                 continue
             args = sequence[2:]
@@ -236,7 +248,7 @@ def main():
                 error_and_exit("Speaker '{}' not found".format(speaker_name))
             if not ap.process_action(speaker, action, args, use_local_speaker_list):
                 error_and_exit(
-                    "Action '{}' not found. \n\nAvailable actions are: {} and 'wait'.\n".format(
+                    "Action '{}' not found. \n\nAvailable actions are: {} and 'wait', 'wait_until'.\n".format(
                         action, list(ap.actions.keys())
                     )
                 )
